/**********************************************************************
 * File Name: SimObject.java
 *
 * Copyright (C) 2000, US Air Force Research Laboratory,
 * Rome Research Site, Rome NY 13424, USA.
 *
 * Use of this software implies acceptance of the terms described in
 * accompanying license.txt file.
 *
 *
 * Authors: Jennifer Dzek, James Lawton, John Crowter,
 *          Bob Paragi, and Joe Beasock
 *
 * Creation Date: July 1999
 *
 * Description:
 *
 * Modification History:
 *
 **********************************************************************/

import java.lang.*;
import java.io.*;
import java.util.*;
import java.awt.*;

public class SimObject {

    String name;
    public SimEnvironment env;
    public ArrayList event_list;
    static int INF = 100000000;

    // Constructors
    public SimObject () {
	name = new String("Unknown");
	event_list = new ArrayList ();
    }

    public SimObject(String newname) {
	name = newname;
	event_list = new ArrayList ();
    }

    // Debugging stuff
    public void print_self() {
	System.out.println("SimObj: " + name);
    }

    // Event list manipulation
    public void schedule ( int event_type, int event_time) {
	if (env.Debug > 1)
	    System.out.println(name + " Scheduling event for time " + event_time);
	EventStruct event = new EventStruct (event_type, event_time);
	insertEvent(event, event_list);
    }

    public void insertEvent(EventStruct ev, ArrayList lst) {
      int i;
      EventStruct v;

      for (i = 0; i < lst.size(); i++) {
	v = (EventStruct)lst.get(i);
	if (ev.event_time < v.event_time)
	  break;
      } // for

      // add the element at the appropriate index
      lst.add(i, ev);

      /*
      if (lst.isEmpty())
	lst.add(ev);
      else {
	EventStruct v;
	int index = 0;
	boolean found = false;
	
	for (ListIterator vals = lst.listIterator(); (! found) && vals.hasNext(); ) {
	  v = (EventStruct) vals.next();
	  if (ev.event_time < v.event_time) {
	    found = true;
	    index = lst.indexOf(v);
	  }
	}

	if (found)
	  lst.add(index, ev);
	else
	  lst.add(ev);
      }
      */
    }


    public void print_events () {
	System.out.println (name +" Event list: " + event_list.toString ());
    }



    public int tick (int simtime) {
      EventStruct next_event;

      while (event_list.size() != 0) {
	next_event = (EventStruct)event_list.get(0);

	// execute the current event, if possible
	if (next_event.event_time == simtime) {
	  dispatchEvent(next_event);
	  event_list.remove(0);
	} // if
	else if (next_event.event_time > simtime)
	  return next_event.event_time;
	else {
	  System.err.println (name + "(" + simtime 
			      + "): Skipped expired event: type = " + 
			      next_event.event_type + 
			      " time = " + next_event.event_time);
	  System.exit(1);
	} // else
      } // while
      
      return INF;

      /*
      int next_event_time;
      boolean more_events;
      EventStruct next_event = null; //new EventStruct();

      // Get the first event
      if ( ! event_list.isEmpty() ) {
	more_events = true;
	next_event = (EventStruct) event_list.get(0);
	next_event_time = next_event.event_time;
      }
      else {
	more_events = false;
	next_event_time = INF;
	return(next_event_time);
      }

      //Execute all events scheduled for 'simtime'
      while (more_events && (next_event_time <= simtime)) {
	
	// Remove the event from the events list:
	event_list.remove(0);

	//Execute the current event, if possible
	if (next_event_time == simtime) {
	  dispatchEvent (next_event);
	}
	else {
	  System.out.println (name + "(" + simtime 
			      + "): Skipped expired event: type = " + 
			      next_event.event_type 
			      + " time = " + next_event.event_time);
	  //System.exit(1);
	}

	// Get the next event
	if ( ! event_list.isEmpty() ) {
	  next_event = (EventStruct) event_list.get(0);
	  next_event_time = next_event.event_time;
	}
	else {
	  more_events = false;
	}
      }
      
      // Return the next event's simulation time
      return (next_event_time);
      */
      }


    // Do something with the event.  Should be overridden by subclass.
    public void dispatchEvent(EventStruct event) {
	System.out.println(" SIMOBJ dispatchEvent: type= " + event.event_type + " Time= " 
			   + event.event_time);
    }

    // Draw the object.  Should be overridden by subclass.
    public void drawSelf(Graphics g, int screen_width, int screen_height) {
    }

    // Shut down object cleanly.  Should be overridden by subclass.
    public void shutdown() {
	//System.out.println("SIMOBJ " + name + " shutting down");
    }
}
